/**
 * Exemple de code pour lire plusieurs capteurs DS18B20 sur un même bus 1-Wire.
 */
 
/* Dépendance pour le bus 1-Wire */
#include <OneWire.h>

/* Broche du bus 1-Wire */
const byte BROCHE_ONEWIRE = 7;

/* Code de retour de la fonction getTemperature() */
enum DS18B20_RCODES {
  READ_OK,
  NO_SENSOR_FOUND,
  INVALID_ADDRESS,
  INVALID_SENSOR
};


/* Création de l'objet OneWire pour manipuler le bus 1-Wire */
OneWire ds(BROCHE_ONEWIRE);

// LiquidCrystal - Version: Latest 
#include <LiquidCrystal.h>

// initialize the library by associating any needed LCD interface pin
// with the arduino pin number it is connected to
const int rs = 12, en = 11, d4 = 5, d5 = 4, d6 = 3, d7 = 2;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

int nbTempMeasures;

const int nbSensors = 3;
float sensorTemp[nbSensors];
byte sensorAddr[nbSensors][8];

/**
 * Fonction de lecture de la température via un capteur DS18B20.
 */
byte getAllTemperatures() {
  byte data[9];
  // data[] : Données lues depuis le scratchpad
  // addr[] : Adresse du module 1-Wire détecté
  
  for (int sensorIndex = 0 ; sensorIndex < nbSensors ; sensorIndex++) {
    /* Reset le bus 1-Wire et sélectionne le capteur */
    ds.reset();
    ds.select(sensorAddr[sensorIndex]);  
    /* Lance une prise de mesure de température et attend la fin de la mesure */
    ds.write(0x44, 1);
  }
  
  delay(800);
  
  for (int sensorIndex = 0 ; sensorIndex < nbSensors ; sensorIndex++) {
    /* Reset le bus 1-Wire, sélectionne le capteur et envoie une demande de lecture du scratchpad */
    ds.reset();
    ds.select(sensorAddr[sensorIndex]);
    ds.write(0xBE);
 
    /* Lecture du scratchpad */
    for (byte i = 0; i < 9; i++) {
      data[i] = ds.read();
    }
   
    /* Calcul de la température en degré Celsius */
    sensorTemp[sensorIndex] = (int16_t) ((data[1] << 8) | data[0]) * 0.0625; 
  }
  // Pas d'erreur
  return READ_OK;
}
 
 
/** Fonction setup() **/
void setup() {

  /* Initialisation du port série */
  Serial.begin(9600);

    // set up the LCD's number of columns and rows:
  lcd.begin(16, 2);
  // Print a message to the LCD.
  lcd.print("Let's go!");

  nbTempMeasures = 0;

  /* Reset le bus 1-Wire ci nécessaire (requis pour la lecture du premier capteur) */
  ds.reset_search();
 
  for (int sensorIndex = 0 ; sensorIndex < nbSensors ; sensorIndex++) {
      /* Recherche le prochain capteur 1-Wire disponible */
      if (!ds.search(sensorAddr[sensorIndex])) {
        // Pas de capteur
        Serial.println("Erreur: capteur introuvable "+sensorIndex);
        return NO_SENSOR_FOUND;
      }
      
      /* Vérifie que l'adresse a été correctement reçue */
      if (OneWire::crc8(sensorAddr[sensorIndex], 7) != sensorAddr[sensorIndex][7]) {
        // Adresse invalide
        Serial.println("Erreur: adresse invalide pour capteur "+sensorIndex);
        return INVALID_ADDRESS;
      }
     
      /* Vérifie qu'il s'agit bien d'un DS18B20 */
      if (sensorAddr[sensorIndex][0] != 0x28) {
        // Mauvais type de capteur
        Serial.println("Erreur: type incorrect pour capteur "+sensorIndex);
        return INVALID_SENSOR;
      }
  }

  Serial.println("CLEARSHEET"); // clears sheet starting at row 1
  Serial.println("LABEL,Time,Counter,Temp1,Temp2,Temp3");
}
 
 
/** Fonction loop() **/
void loop() {
  if (nbTempMeasures >= 99999)
    nbTempMeasures = 1;
  else
    nbTempMeasures++;

  getAllTemperatures();

  /* Affiche les températures */
  String line1 = "";
  line1 += nbTempMeasures;
  line1 += ",";
  line1 += String(sensorTemp[0], 2);
  line1 += ",";
  String line2 = String(sensorTemp[1], 2);
  line2 += ",";
  line2 += String(sensorTemp[2], 2);
  Serial.println( (String) "DATA,TIME," + line1 + line2);

  lcd.clear();
  lcd.print("#"+line1);
  lcd.setCursor(1,2);
  lcd.print(line2);
}
